/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#include "init.h"
#include "CNC_functions.h"
#include "CNC_settings.h"
#include "CNC_menu.h"

#include <intEEPROM.h>
#include <intEEPROM.c>


extern void RS485_show_line (void);

//**********************************************
//               system values
//**********************************************


unsigned char system_Input_Voltage_Max = 0 ;
unsigned char system_Input_Voltage_Min = 0 ;
unsigned char system_Delta_V_Warning = 0 ;
unsigned char system_Delta_V_Error = 0 ;

unsigned char system_PWM = 0;

unsigned char system_Temeperature_Fan = 0;
unsigned char system_Temeperature_Warning = 0;
unsigned char system_Temeperature_Error = 0;


//**********************************************
//          CNC_system_initialization
//**********************************************

void CNC_system_initialization (void) {

    initialize_RS485 (BAUD_RATE_115200,RX_INTERRUPT_ON, TX_INTERRUPT_OFF);

#ifdef CNC_MENU

    // Sending a Clear Let the RS232 Terminal clear the Input buffer
    // This feature is useful for the menu it is like a clear monitor instruction.
    send_delete_RS485 ();

    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("       Stepper Motor Controller ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();

#endif CNC_MENU
    
    load_parameters_from_EEPROM ();

    activate_signal_interupt (ENABLE_SIGNAL, HIGH_LEVEL_PRIORITY);
    activate_signal_interupt (STEP_SIGNAL, HIGH_LEVEL_PRIORITY);
    activate_signal_interupt (DIRECTION_SIGNAL, HIGH_LEVEL_PRIORITY);
    activate_signal_interupt (SWITCH_SIGNAL, LOW_LEVEL_PRIORITY);

    system_voltage_start = ADC_read_VIN_TEST ();

    IPR1bits.ADIP = LOW_LEVEL_PRIORITY;
    PIE1bits.ADIE = ACTIVATED;

//    write_const_string_RS485 (" Interrupt Signals initialized.");
//    send_return_RS485 ();

    initialize_TIMER0 (LOW_LEVEL_PRIORITY);

//    write_const_string_RS485 (" Timer Scheduler activated.");
//    send_return_RS485 ();

    initialize_interrupt (ACTIVATED);

//    write_const_string_RS485 (" Interrupts activated.");
//    send_return_RS485 ();

    initialize_stepper_motor (HALF_STEP, system_PWM - 1, MAX_PHASE_CURRENT_PEAK);

//    write_const_string_RS485 (" Stepper Motor initialized.");
//    send_return_RS485 ();

//    write_const_string_RS485 (" State Machine Running...");
//    send_return_RS485 ();
//    send_return_RS485 ();


    reference_phase_current = I_peak_current_table[get_dip_switch_value ()];
    max_phase_current = MAX_PHASE_CURRENT_PEAK;
    
#ifdef CNC_MENU
    send_return_RS485 ();
    write_const_string_RS485 (" Stepper Motor Ipeak : ");    
    write_int_ASCII_RS485 (reference_phase_current * 10, 5);
    write_const_string_RS485 (" mA");
    send_return_RS485 ();

    write_const_string_RS485 (" Stepper Motor Imax Peak: ");
    write_int_ASCII_RS485 (max_phase_current * 10, 5);
    write_const_string_RS485 (" mA");
    send_return_RS485 ();

    write_const_string_RS485 (" Stepper Motor PWM : ");
    write_int_ASCII_RS485 (system_PWM, 3);
    write_const_string_RS485 (" KHz");
    send_return_RS485 ();

    write_const_string_RS485 (" System Temperature [F] : ");
    write_int_ASCII_RS485 (system_Temeperature_Fan, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();

    write_const_string_RS485 (" System Temperature [W] : ");
    write_int_ASCII_RS485 (system_Temeperature_Warning, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();

    write_const_string_RS485 (" System Temperature [E] : ");
    write_int_ASCII_RS485 (system_Temeperature_Error, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();
    
    write_const_string_RS485 (" System Starting Voltage : ");
    write_int_ASCII_RS485 (system_voltage_start, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" Input Voltage Max. : ");
    write_int_ASCII_RS485 (system_Input_Voltage_Max, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" Input Voltage Min. : ");
    write_int_ASCII_RS485 (system_Input_Voltage_Min, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" System Voltage Delta [W]: ");
    write_int_ASCII_RS485 (system_Delta_V_Warning, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" System Voltage Delta [E]: ");
    write_int_ASCII_RS485 (system_Delta_V_Error, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();
    send_return_RS485 ();

    //write_const_string_RS485 ("----------------------------------------");
    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 (" Press 'm' to enter the Menu': ");
#endif CNC_MENU
}

//**********************************************
//          CNC_state_voltage_measure
//**********************************************
void load_parameters_from_EEPROM (void) {
    
    unsigned char data = 0;

    data = read_internal_EEPROM (ADD_FIRST_BOOTING);
    if (data == FIRST_BOOT_VALUE ) {

        #ifdef CNC_MENU
            write_const_string_RS485 (" EEPROM already Initialized.");
        #endif

        system_Input_Voltage_Max = read_internal_EEPROM (ADD_VOLTAGE_MAX);
        system_Input_Voltage_Min = read_internal_EEPROM (ADD_VOLTAGE_MIN);
        system_Delta_V_Warning = read_internal_EEPROM (ADD_DELTA_VOLTAGE_WARNING);
        system_Delta_V_Error = read_internal_EEPROM (ADD_DELTA_VOLTAGE_ERROR);
        system_PWM  = read_internal_EEPROM (ADD_PWM);
        system_Temeperature_Fan = read_internal_EEPROM (ADD_TEMPERATURE_FAN);
        system_Temeperature_Warning = read_internal_EEPROM (ADD_TEMPERATURE_WARNING);
        system_Temeperature_Error = read_internal_EEPROM (ADD_TEMPERATURE_ERROR);


    } else {
        #ifdef CNC_MENU
            write_const_string_RS485 (" First Board booting. Default Values will be loaded.");
        #endif

        write_internal_EEPROM (ADD_FIRST_BOOTING, FIRST_BOOT_VALUE);

        write_internal_EEPROM (ADD_VOLTAGE_MAX, VIN_MAX);
        write_internal_EEPROM (ADD_VOLTAGE_MIN, VIN_MIN);
        write_internal_EEPROM (ADD_DELTA_VOLTAGE_WARNING, VIN_DELTA_WARNING);
        write_internal_EEPROM (ADD_DELTA_VOLTAGE_ERROR, VIN_DELTA_ERROR);
        write_internal_EEPROM (ADD_PWM, DEFAULT_PWM);
        write_internal_EEPROM (ADD_TEMPERATURE_FAN, TEMP_WARNING_1);
        write_internal_EEPROM (ADD_TEMPERATURE_WARNING, TEMP_WARNING_2);
        write_internal_EEPROM (ADD_TEMPERATURE_ERROR, TEMP_MAX);

        system_Input_Voltage_Max = VIN_MAX;
        system_Input_Voltage_Min = VIN_MIN;
        system_Delta_V_Warning = VIN_DELTA_WARNING;
        system_Delta_V_Error = VIN_DELTA_ERROR;
        system_PWM = DEFAULT_PWM;

        system_Temeperature_Fan = TEMP_WARNING_1;
        system_Temeperature_Warning = TEMP_WARNING_2;
        system_Temeperature_Error = TEMP_MAX;

    } 


}


//**********************************************
//          CNC_state_voltage_measure
//**********************************************
void CNC_state_voltage_measure (void) {
    // Abort the current ADC Measure
    ADCON0bits.ADON = 0;

    // Select Group B
    ADCON0bits.ACMOD0 = 1;
    ADCON0bits.ACMOD1 = 0;

    //Select AN5 Channel within the group B
    ADCHS =  0b00010000;

    ADC_State = ADC_VIN_TEST;

    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    voltage_measure_request = DEACTIVATED;

    ADC_running = ADC_IS_BUSY;
}


//**********************************************
//          CNC_state_voltage_measure
//**********************************************
void CNC_state_temperature_measure (void) {
    // Abort the current ADC Measure
        ADCON0bits.ADON = 0;

        // Select Group A
	ADCON0bits.ACMOD0 = 0;
	ADCON0bits.ACMOD1 = 0;

	//Select AN8 Channel within the group A
	ADCHS =  0b00000010;

        ADC_State = ADC_TEMP_TEST;

        ADCON0bits.ADON = 1;
        ADCON0bits.GO_DONE = 1;

        temperature_measure_request = DEACTIVATED;

        ADC_running = ADC_IS_BUSY;
}


//**********************************************
//          CNC_state_system_monitor
//**********************************************
void CNC_state_system_monitor (void){

    if (system_voltage < system_Input_Voltage_Min){
        set_error_status (ERROR_UNDER_VOLTAGE);
    }

    if (system_voltage > system_Input_Voltage_Max){
        set_error_status (ERROR_OVER_VOLTAGE);
    }

    // Check if the input voltage did not change VIN_DELTA_WARNING during operation
    if (system_voltage != 0) {
      if ((system_voltage - system_voltage_start) > system_Delta_V_Warning) {
        set_warning_status (WARNING_DELTA_VOLTAGE_O);
        } else if ((system_voltage_start - system_voltage) > system_Delta_V_Warning) {
            set_warning_status (WARNING_DELTA_VOLTAGE_B);
        }
    }

    // Check if the input voltage did not change VIN_DELTA_ERROR during operation
    if (system_voltage != 0) {
      if ((system_voltage - system_voltage_start) > system_Delta_V_Error) {
        set_error_status (ERROR_DELTA_VOLTAGE_O);
        } else if ((system_voltage_start - system_voltage) > system_Delta_V_Error) {
            set_error_status (ERROR_DELTA_VOLTAGE_B);
        }
    }
    

    if (system_temperature > system_Temeperature_Error){
            turn_on_FAN ();
            set_error_status (ERROR_OVER_TEMP);
    }

    if (system_temperature > system_Temeperature_Fan){
            turn_on_FAN ();
    } else
        if (system_temperature < system_Temeperature_Fan - TEMP_HYSTERESIS)
            turn_off_FAN ();

    if (system_temperature > system_Temeperature_Warning)
        set_warning_status (WARNING_TEMPERATURE);

    sytem_monitor_request = DEACTIVATED;
}


//**********************************************
//          CNC_state_motor_1_current_test
//**********************************************
void CNC_state_phase_1_current_read (void) {

    // Current is measured only in case the phase is active

    if (IN1_HB_1 != DEACTIVATED || IN2_HB_1 != DEACTIVATED) {
        phase_1_current_sum += ADC_buffer_reading;
        phase_1_current_read++;

        if (phase_1_current_read == CURRENT_AVARAGING_SAMPLES) {
            phase_1_current = phase_1_current_sum >> CURRENT_AVARAGING_SHIFT;
            phase_1_current_read = 0;
            phase_1_current_sum = 0;

            CNC_current_control_loop ();
        }
    }
    

    // Select Motor 2
    // Select Group B
    ADCON0bits.ACMOD0 = 1;
    ADCON0bits.ACMOD1 = 0;

    //Select AN1 Channel within the group B
    ADCHS =  0b00000000;

    ADC_State = ADC_PHASE_2_TEST;

    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    ADC_running = ADC_IS_BUSY;
}

//**********************************************
//          CNC_state_motor_2_current_read
//**********************************************
void CNC_state_phase_2_current_read (void) {

    // Current is measured only in case the phase is active

    if (IN1_HB_2 != DEACTIVATED || IN2_HB_2 != DEACTIVATED) {
        phase_2_current_sum += ADC_buffer_reading;
        phase_2_current_read++;

        if (phase_2_current_read == CURRENT_AVARAGING_SAMPLES) {
            phase_2_current = phase_2_current_sum >> CURRENT_AVARAGING_SHIFT;
            phase_2_current_read = 0;
            phase_2_current_sum = 0;

            CNC_current_control_loop ();
        }

    }
    
    // Select Motor 1
    // Select Group A
    ADCON0bits.ACMOD0 = 0;
    ADCON0bits.ACMOD1 = 0;

    //Select AN0 Channel within the group A
    ADCHS =  0b00000000;

    ADC_State = ADC_PHASE_1_TEST;

    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    ADC_running = ADC_IS_BUSY;
}


//**********************************************
//         CNC_state_Vin_read
//**********************************************
void CNC_state_Vin_read (void){

    system_voltage_sum += ADC_buffer_reading/23;
    system_voltage_read++;

    if (system_voltage_read == VOLTAGE_AVARAGING_SAMPLES) {
        system_voltage = system_voltage_sum >> VOLTAGE_AVARAGING_SHIFT;
        system_voltage_read = 0;
        system_voltage_sum = 0;
    }

    // Select Motor 1
    // Select Group A
    ADCON0bits.ACMOD0 = 0;
    ADCON0bits.ACMOD1 = 0;

    //Select AN0 Channel within the group A
    ADCHS =  0b00000000;

    ADC_State = ADC_PHASE_1_TEST;

    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    ADC_running = ADC_IS_BUSY;
}

//**********************************************
//         CNC_state_temperature_read
//**********************************************
void CNC_state_temperature_read (void) {

    system_temperature_sum += ADC_buffer_reading >> 1;
    system_temperature_read++;

    if (system_temperature_read == TEMPERATURE_AVARAGING_SAMPLES) {
        system_temperature = system_temperature_sum >> TEMPERATURE_AVARAGING_SHIFT;
        system_temperature_read = 0;
        system_temperature_sum = 0;
    }

    // Select Motor 1
    // Select Group A
    ADCON0bits.ACMOD0 = 0;
    ADCON0bits.ACMOD1 = 0;

    //Select AN0 Channel within the group A
    ADCHS =  0b00000000;

    ADC_State = ADC_PHASE_1_TEST;

    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    ADC_running = ADC_IS_BUSY;
}



//**********************************************
//         CNC_control_loop_P
//**********************************************
void CNC_current_control_loop (void){

// The control Loop is executed only if no error is active

    if (error_status == 0) {


//    if (reference_phase_current > phase_1_current) {
//        error_phase_1 = reference_phase_current - phase_1_current;
//        phase_1_controlled_PWM += error_phase_1 >> FILTER_KP;
//    }
//    else {
//        error_phase_1 = phase_1_current - reference_phase_current;
//        phase_1_controlled_PWM -= error_phase_1 >> FILTER_KP;
//    }
//
//
//    if (reference_phase_current > phase_2_current){
//        error_phase_2 = reference_phase_current - phase_2_current;
//        phase_2_controlled_PWM += error_phase_2 >> FILTER_KP;
//    }
//    else {
//        error_phase_2 = phase_2_current - reference_phase_current;
//        phase_2_controlled_PWM -= error_phase_2 >> FILTER_KP;
//    }
    

   if (reference_phase_current > phase_1_current) {
        phase_1_controlled_PWM += FILTER_KP;
    }
    else {
        phase_1_controlled_PWM -= FILTER_KP;
    }


    if (reference_phase_current > phase_2_current){
        phase_2_controlled_PWM += FILTER_KP;
    }
    else {
        phase_2_controlled_PWM -= FILTER_KP;
    }



    // if Phase current is 0mA the PWM is set to 0 bypassing the loop

    if (reference_phase_current == 0) {
        phase_1_controlled_PWM = 0;
        phase_2_controlled_PWM = 0;
    }


   //This control is required to void PWM resoltion overflow
   //It should be otimized depending to the PWM resolution
   //TO DO LIST
   if (phase_1_controlled_PWM > 128) {
       phase_1_controlled_PWM = 128;
   }

   if (phase_2_controlled_PWM > 128) {
       phase_2_controlled_PWM = 128;
   }


    SetDCPWM1 (phase_1_controlled_PWM);
    SetDCPWM2 (phase_2_controlled_PWM);

}  

}