/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#include "CNC_menu.h"
#include <intEEPROM.h>


extern unsigned char system_Input_Voltage_Max;
extern unsigned char system_Input_Voltage_Min;
extern unsigned char system_Delta_V_Warning;
extern unsigned char system_Delta_V_Error;

extern unsigned char system_PWM;

extern unsigned char system_Temeperature_Fan;
extern unsigned char system_Temeperature_Warning;
extern unsigned char system_Temeperature_Error;

// This variables are used to keep the digit counter
// Once the voltage/pwm/temperature are sent via RS485
unsigned char voltage_id_counter;
unsigned char voltage_buffer;

unsigned char pwm_id_counter;
unsigned char pwm_buffer;

unsigned char temperature_id_counter;
unsigned char temperature_buffer;


void RS485_show_line (void) {
   write_const_string_RS485 ("----------------------------------------");
}


//**********************************************
//              CNC_check_menu
//**********************************************
void CNC_check_menu (unsigned char input_data) {


#ifdef CNC_MENU

    if(input_data == 'm' && menu_level == MENU_LEVEL_0) {
            CNC_show_menu_1 ();
    }



//**********************************************
//      Catch data limit inputs
//**********************************************

    // Check if the input is a number
    // Afterard it checks the menu' level
    if (input_data> 47 && input_data< 58) {

        switch (menu_level) {

     //**********************************************
     //      Catch Voltage limit inputs
     //**********************************************

            case MENU_LEVEL_4 :
                                 write_internal_EEPROM (ADD_DELTA_VOLTAGE_ERROR, input_data - 48);
                                 system_Delta_V_Error = input_data-48;
                                 CNC_show_menu_voltage_limits_2 ();
                                 break;

            case MENU_LEVEL_5 :

                                 write_internal_EEPROM (ADD_DELTA_VOLTAGE_WARNING, input_data - 48);
                                 system_Delta_V_Warning = input_data-48;
                                 CNC_show_menu_voltage_limits_2 ();
                                 break;

            case MENU_LEVEL_6 :
                                voltage_id_counter++;

                                if (voltage_id_counter == 1){
                                    voltage_buffer += (input_data - 48) * 10;
                                } else {
                                      voltage_buffer += (input_data - 48);
                                      write_internal_EEPROM (ADD_VOLTAGE_MAX, voltage_buffer);
                                      system_Input_Voltage_Max = voltage_buffer;
                                      CNC_show_menu_voltage_limits_2 ();
                                 }
                                break;


            case MENU_LEVEL_7 :
                                voltage_id_counter++;

                                if (voltage_id_counter == 1){
                                    voltage_buffer += (input_data - 48) * 10;
                                } else {
                                      voltage_buffer += (input_data - 48);
                                      write_internal_EEPROM (ADD_VOLTAGE_MIN, voltage_buffer);
                                      system_Input_Voltage_Min = voltage_buffer;
                                      CNC_show_menu_voltage_limits_2 ();
                                 }
                                break;

            //**********************************************
            //      Catch Temperature limit inputs
            //**********************************************
            case MENU_LEVEL_11 : temperature_id_counter++;

                                if (temperature_id_counter == 1){
                                    temperature_buffer += (input_data - 48) * 10;
                                } else {
                                      temperature_buffer += (input_data - 48);
                                      write_internal_EEPROM (ADD_TEMPERATURE_FAN, temperature_buffer);
                                      system_Temeperature_Fan = temperature_buffer;
                                      CNC_show_menu_Temperature_10 ();
                                 }
                                break;

            case MENU_LEVEL_12  : temperature_id_counter++;

                                if (temperature_id_counter == 1){
                                    temperature_buffer += (input_data - 48) * 10;
                                } else {
                                      temperature_buffer += (input_data - 48);
                                      write_internal_EEPROM (ADD_TEMPERATURE_WARNING, temperature_buffer);
                                      system_Temeperature_Warning = temperature_buffer;
                                      CNC_show_menu_Temperature_10 ();
                                 }
                                break;


            case MENU_LEVEL_13  : temperature_id_counter++;

                                if (temperature_id_counter == 1){
                                    temperature_buffer += (input_data - 48) * 10;
                                } else {
                                      temperature_buffer += (input_data - 48);
                                      write_internal_EEPROM (ADD_TEMPERATURE_ERROR, temperature_buffer);
                                      system_Temeperature_Error = temperature_buffer;
                                      CNC_show_menu_Temperature_10 ();
                                 }
                                break;

            //**********************************************
            //      Catch PWM value
            //**********************************************
             case MENU_LEVEL_9  :  pwm_id_counter++;

                                    if (pwm_id_counter == 1){
                                        pwm_buffer += (input_data - 48) * 10;
                                    } else {
                                          pwm_buffer += (input_data - 48);
                                          write_internal_EEPROM (ADD_PWM, pwm_buffer);
                                          system_PWM = pwm_buffer;
                                          //initialize_stepper_motor (HALF_STEP, system_PWM - 1, MAX_PHASE_CURRENT_PEAK);
                                          CNC_show_menu_PWM_Frequency_8 ();
                                     }
                                    break;
        }

    }

    //**********************************************
    //               CNC_show_menu_1
    //**********************************************
    if(input_data == '1' && menu_level == MENU_LEVEL_1) {
       CNC_show_menu_voltage_limits_2 ();
    }

    if(input_data == '2' && menu_level == MENU_LEVEL_1) {
       CNC_show_menu_Temperature_10 ();
    }

    if(input_data == '3' && menu_level == MENU_LEVEL_1) {
       CNC_show_menu_PWM_Frequency_8 ();
    }


    if(input_data == '4' && menu_level == MENU_LEVEL_1) {
       CNC_show_menu_System_Warning_Error_3 ();
    }


    //**********************************************
    //   CNC_show_menu_System_Error_Warning_3
    //**********************************************
    if(input_data == 'a' && menu_level == MENU_LEVEL_3) {
       CNC_show_menu_System_Warning_Error_3 ();
    }

    if(input_data == 'b' && menu_level == MENU_LEVEL_3) {
        warning_status = 0;
        CNC_show_menu_System_Warning_Error_3 ();
    }

    if(input_data == 'e' && menu_level == MENU_LEVEL_3) {
       CNC_show_menu_1 ();
    }
    
    //**********************************************
    //      CNC_show_menu_voltage_limits_2
    //**********************************************

    if(input_data == 'a' && menu_level == MENU_LEVEL_2) {
            CNC_show_menu_Voltage_Max_6 ();
    }

    if(input_data == 'b' && menu_level == MENU_LEVEL_2) {
            CNC_show_menu_Voltage_Min_7 ();
    }

    if(input_data == 'c' && menu_level == MENU_LEVEL_2) {
            CNC_show_menu_Delta_Voltage_Warning_5 ();
    }

    if(input_data == 'd' && menu_level == MENU_LEVEL_2) {
            CNC_show_menu_Delta_Voltage_Error_4 ();
    }

    if(input_data == 'e' && menu_level == MENU_LEVEL_2) {
            CNC_show_menu_1 ();
    }

    //**********************************************
    //      CNC_show_menu_PWM_Frequency_8
    //**********************************************

    if(input_data == 'a' && menu_level == MENU_LEVEL_8) {
            CNC_show_menu_set_PWM_Frequency_9 ();
    }

    if(input_data == 'e' && menu_level == MENU_LEVEL_8) {
            CNC_show_menu_1 ();
    }

    //**********************************************
    //      CNC_show_menu_temperature_10
    //**********************************************

    if(input_data == 'a' && menu_level == MENU_LEVEL_10) {
            CNC_show_menu_Temperature_Fan_11 ();
    }
    
    if(input_data == 'b' && menu_level == MENU_LEVEL_10) {
            CNC_show_menu_Temperature_Warning_12 ();
    }
    
    if(input_data == 'c' && menu_level == MENU_LEVEL_10) {
            CNC_show_menu_Temperature_Error_13 ();
    }

    if(input_data == 'e' && menu_level == MENU_LEVEL_10) {
            CNC_show_menu_1 ();
    }


#endif CNC_MENU
}


//**********************************************
//         CNC_show_menu
//**********************************************
void CNC_show_menu_1 () {

    // Sending a Clear Let the RS232 Terminal clear the Input buffer
    // This feature is useful for the menu it is like a clear monitor instruction.
    send_delete_RS485 ();

    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("         Configuration Menu' ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" 1) System Voltage limits");
    send_return_RS485 ();
    write_const_string_RS485 (" 2) System Temperature limits");
    send_return_RS485 ();
    write_const_string_RS485 (" 3) PWM Frequency");
    send_return_RS485 ();
    write_const_string_RS485 (" 4) System Warnings and Errors");
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" Select an option : ");

    menu_level = MENU_LEVEL_1;

}


//**********************************************
//         CNC_show_menu_voltage_limits
//**********************************************

void CNC_show_menu_voltage_limits_2 () {

    // Sending a Clear Let the RS232 Terminal clear the Input buffer
    // This feature is useful for the menu it is like a clear monitor instruction.
    send_delete_RS485 ();

    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("         Voltage Limits Menu' ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();

    write_const_string_RS485 (" a) System Vmax       : ");
    write_int_ASCII_RS485 (system_Input_Voltage_Max, 3);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" b) System Vmin       : ");
    write_int_ASCII_RS485 (system_Input_Voltage_Min, 3);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" c) Delta Vin Warning : ");
    write_int_ASCII_RS485 (system_Delta_V_Warning, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" d) Delta Vin Error   : ");
    write_int_ASCII_RS485 (system_Delta_V_Error, 4);
    write_const_string_RS485 (" V");
    send_return_RS485 ();

    write_const_string_RS485 (" e) Esc");
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" Select an option : ");

    menu_level = MENU_LEVEL_2;

}



//**********************************************
//         CNC_show_menu_voltage_limits
//**********************************************

void CNC_show_menu_System_Warning_Error_3 (void) {
    // Sending a Clear Let the RS232 Terminal clear the Input buffer
    // This feature is useful for the menu it is like a clear monitor instruction.
    send_delete_RS485 ();

    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("        System Warnings and Errors ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();

    write_const_string_RS485 (" *) WARNING Missing Steps   : ");
    if (warning_status & WARNING_MISSING_STEPS) {
        write_const_string_RS485 ("ACTIVATED");
    } else {
        write_const_string_RS485 ("-");
    }
    send_return_RS485 ();


    write_const_string_RS485 (" *) WARNING Delta Over Vin  : ");
    if (warning_status & WARNING_DELTA_VOLTAGE_O) {
        write_const_string_RS485 ("ACTIVATED");
    } else {
        write_const_string_RS485 ("-");
    }
    send_return_RS485 ();


    write_const_string_RS485 (" *) WARNING Delta Below Vin : ");
    if (warning_status & WARNING_DELTA_VOLTAGE_B) {
        write_const_string_RS485 ("ACTIVATED");
    } else {
        write_const_string_RS485 ("-");
    }
    send_return_RS485 ();


    write_const_string_RS485 (" *) WARNING Temperature     : ");
    if (warning_status & WARNING_TEMPERATURE) {
        write_const_string_RS485 ("ACTIVATED");
    } else {
        write_const_string_RS485 ("-");
    }
    send_return_RS485 ();

    write_const_string_RS485 (" *) WARNING RS485           : ");
    if (warning_status & WARNING_COMMUNICATION) {
        write_const_string_RS485 ("ACTIVATED");
    } else {
        write_const_string_RS485 ("-");
    }

    send_return_RS485 ();
    send_return_RS485 ();

    write_const_string_RS485 (" *) Error Code : ");
    if (error_status != 0) {
            write_int_ASCII_RS485 (error_status, 4);
    } else {
        write_const_string_RS485 ("-");
    }   

    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" *) Missed Steps Counter: ");
    write_int_ASCII_RS485 (missing_steps_counter, 4);


    send_return_RS485 ();
    send_return_RS485 ();
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" a) Refresh Data");
    send_return_RS485 ();
    write_const_string_RS485 (" b) Clear Warning Register");
    send_return_RS485 ();
    write_const_string_RS485 (" e) Esc");
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" Select an option : ");

    menu_level = MENU_LEVEL_3;

}

//**********************************************
//     CNC_show_voltage_input_1_9
//**********************************************

void CNC_show_voltage_input_1_9 (void) {
    send_delete_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" New Value [1-9]: ");
}

//**********************************************
//     CNC_show_menu_Delta_Voltage_Error_4
//**********************************************

void CNC_show_menu_Delta_Voltage_Error_4 (void) {

    CNC_show_voltage_input_1_9 ();
    menu_level = MENU_LEVEL_4;
}


//**********************************************
//     CNC_show_menu_Delta_Voltage_Warning_5
//**********************************************
void CNC_show_menu_Delta_Voltage_Warning_5 (void) {

    CNC_show_voltage_input_1_9 ();
    menu_level = MENU_LEVEL_5;
}


//**********************************************
//     CNC_show_voltage_input_12_28
//**********************************************
void CNC_show_voltage_input_12_28 (void) {
    send_delete_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" New Value [12-28]: ");

    //initialize the variable to start receving new values
    voltage_id_counter = 0;
    voltage_buffer = 0;
}


//**********************************************
//     CNC_show_menu_Voltage_Max_6
//**********************************************
void CNC_show_menu_Voltage_Max_6 (void) {

    CNC_show_voltage_input_12_28 ();
    menu_level = MENU_LEVEL_6;
}


//**********************************************
//     CNC_show_menu_Voltage_Min_7
//**********************************************
void CNC_show_menu_Voltage_Min_7 (void) {

    CNC_show_voltage_input_12_28 ();
    menu_level = MENU_LEVEL_7;
}


//**********************************************
//     CNC_show_menu_PWM_Frequency_8
//**********************************************
void CNC_show_menu_PWM_Frequency_8 (void) {
    
    send_delete_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("         PWM Frequency Menu' ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();

    write_const_string_RS485 (" a) PWM Frequency : ");
    write_int_ASCII_RS485 (system_PWM, 3);
    write_const_string_RS485 (" KHz");
    send_return_RS485 ();


    write_const_string_RS485 (" e) Esc");
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" Select an option : ");

    menu_level = MENU_LEVEL_8;
}

//**********************************************
//     CNC_show_menu_set_PWM_Frequency_9
//**********************************************
void CNC_show_menu_set_PWM_Frequency_9 (void) {
    
    send_delete_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" New Value [10-50]: ");

    //initialize the variable to start receving new values
    pwm_id_counter = 0;
    pwm_buffer = 0;

    menu_level = MENU_LEVEL_9;
}

//**********************************************
//     CNC_show_menu_temperature_10
//**********************************************
void CNC_show_menu_Temperature_10 (void) {

    // Sending a Clear Let the RS232 Terminal clear the Input buffer
    // This feature is useful for the menu it is like a clear monitor instruction.
    send_delete_RS485 ();

    RS485_show_line ();
    send_return_RS485 ();
    write_const_string_RS485 ("       Temperature Limits Menu' ");
    send_return_RS485 ();
    RS485_show_line ();
    send_return_RS485 ();
    send_return_RS485 ();

    write_const_string_RS485 (" a) Temperature Fan     : ");
    write_int_ASCII_RS485 (system_Temeperature_Fan, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();

    write_const_string_RS485 (" b) Temperature Warning : ");
    write_int_ASCII_RS485 (system_Temeperature_Warning, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();

    write_const_string_RS485 (" c) Temperature Error   : ");
    write_int_ASCII_RS485 (system_Temeperature_Error, 4);
    write_const_string_RS485 (" C");
    send_return_RS485 ();

    write_const_string_RS485 (" e) Esc");
    send_return_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" Select an option : ");

    menu_level = MENU_LEVEL_10;
}

void CNC_show_temeprature_input (void) {
     send_delete_RS485 ();
    send_return_RS485 ();
    write_const_string_RS485 (" New Value [10-80]: ");

    //initialize the variable to start receving new values
    temperature_id_counter = 0;
    temperature_buffer = 0;

}

//**********************************************
//     CNC_show_menu_Temperature_Fan_11
//**********************************************
void CNC_show_menu_Temperature_Fan_11 (void) {

    CNC_show_temeprature_input ();
    menu_level = MENU_LEVEL_11;
}


//**********************************************
//     CNC_show_menu_Temperature_Warning_12
//**********************************************
void CNC_show_menu_Temperature_Warning_12 (void){

    CNC_show_temeprature_input ();
    menu_level = MENU_LEVEL_12;
}


//**********************************************
//     CNC_show_menu_Temperature_Error_13
//**********************************************
void CNC_show_menu_Temperature_Error_13 (void){

    CNC_show_temeprature_input ();

    menu_level = MENU_LEVEL_13;
}