/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifndef CNC_SETTINGS_H
#define	CNC_SETTINGS_H

#define PHASE_CURRENT 50

// Motor max Current counter
#define MAX_CURRENT_FILTER_TICKS 6

#define MAX_MISSING_STEPS 4

// Task Time Scheduling
#define LED_TICKS 50
#define VOLTAGE_TICKS 10
#define SYSTEM_MONITOR_TICKS 60
#define TEMPERATURE_TICKS 120

// Current Read Avareging
#define CURRENT_AVARAGING_SAMPLES 16
#define CURRENT_AVARAGING_SHIFT 4

// System Voltage Read Avareging
#define VOLTAGE_AVARAGING_SAMPLES 1
#define VOLTAGE_AVARAGING_SHIFT 0

// System Temperature Read Avareging
#define TEMPERATURE_AVARAGING_SAMPLES 4
#define TEMPERATURE_AVARAGING_SHIFT 2

// Anti dump filter for Auxiliary SW1 and SW2
#define AUX_SWITCH_FILTER_TICKS 4

//*********************************************
//    EEPROM Internal parameter addresses
//*********************************************

#define FIRST_BOOT_VALUE 0xAA
#define ADD_FIRST_BOOTING 0
#define ADD_VOLTAGE_MAX 1
#define ADD_VOLTAGE_MIN 2
#define ADD_DELTA_VOLTAGE_WARNING 3
#define ADD_DELTA_VOLTAGE_ERROR 4
#define ADD_PWM 5
#define ADD_TEMPERATURE_FAN 6
#define ADD_TEMPERATURE_WARNING 7
#define ADD_TEMPERATURE_ERROR 8


volatile unsigned char switch_1_pressed_flag = 0;
volatile unsigned char switch_1_filter_counter = 0;

volatile unsigned char switch_2_pressed_flag = 0;
volatile unsigned char switch_2_filter_counter = 0;

volatile unsigned char step_status = DEACTIVATED;
volatile unsigned char enable_status = DEACTIVATED;
volatile unsigned char direction_status = DEACTIVATED;

volatile int ADC_buffer_reading = 0;

// Motor 1 Current Reading
volatile int phase_1_current = 0;
volatile int phase_1_current_sum = 0;
volatile unsigned char phase_1_current_read  = 0;
volatile unsigned char max_current_phase_1_counter = 0;


// Motor 2 Current Reading
volatile int phase_2_current = 0;
volatile int phase_2_current_sum = 0;
volatile unsigned char phase_2_current_read  = 0;
volatile unsigned char max_current_phase_2_counter = 0;

// Voltage Reading
volatile int system_voltage = 0;
volatile int system_voltage_start = 0;
volatile int system_voltage_sum = 0;
volatile unsigned char system_voltage_read = 0;

// Temperature Reading
volatile int system_temperature = 0;
volatile int system_temperature_sum = 0;
volatile unsigned char system_temperature_read = 0;


volatile unsigned char ADC_State = 0;
volatile unsigned char ADC_running = 0;

volatile unsigned char voltage_measure_request = 0;
volatile unsigned char temperature_measure_request = 0;
volatile unsigned char sytem_monitor_request = 0;

volatile int reference_phase_current = 0;
volatile int max_phase_current = 0;

#define FILTER_KP 1

volatile int error_phase_1 = 0;
volatile int error_phase_2 = 0;

// Current set with the calling function
volatile int phase_1_controlled_PWM = 0;
volatile int phase_2_controlled_PWM = 0;

volatile unsigned char missing_steps_counter = 0;

#endif	/* CNC_SETTINGS_H */

